@@ -1,4 +1,4 @@
-from typing import List, Iterator
+from typing import List, Iterator, Dict
 
 from bugzoo.core.patch import Patch
 
@@ -45,9 +45,9 @@ class Candidate(object):
         Computes a plaintext diff for this repair.
         """
         # group transformations by file
-        tf = {}
+        tf = {} # type: Dict[str, List[Transformation]]
         for t in self.transformations:
-            fn = t.line.filename
+            fn = t.char_range.filename # type: ignore
             if fn not in tf:
                 tf[fn] = []
             tf[fn].append(t)
@@ -55,27 +55,27 @@ class Candidate(object):
         # order each group of transformations in descending order of
         # line number
         for group in tf.values():
-            group.sort(key=lambda t: t.line.num, reverse=True)
+            group.sort(key=lambda t: t.char_range.start.offset, reverse=True) # type: ignore
 
         # transform each group into a modified source code file
-        transformed = {}
+        transformed = problem.sources
         for (fn, transformations) in tf.items():
-            src = problem.source(fn)
             for t in transformations:
                 if isinstance(t, DeleteTransformation):
-                    src = src.with_line_removed(t.line.num)
+                    transformed = transformed.delete(t.char_range)
                 elif isinstance(t, AppendTransformation):
-                    src = src.with_line_inserted(t.line.num + 1, t.snippet.content)
+                    transformed = transformed.insert(t.char_range.stop, str(t.snippet))
                 elif isinstance(t, ReplaceTransformation):
-                    src = src.with_line_replaced(t.line.num, t.snippet.content)
+                    transformed = transformed.replace(t.char_range, str(t.snippet))
                 else:
                     raise Exception("unsupported transformation")
-            transformed[fn] = src
 
+        # FIXME use SourceFileCollection.diff
         # compute a diff for each modified source code file
         diffs = []
-        for (fn, source_after) in transformed.items():
-            source_before = problem.source(fn)
+        for fn in transformed:
+            source_after = transformed[fn]
+            source_before = problem.sources[fn]
             diffs.append(source_before.diff(source_after))
 
         # combine the diffs for each file into a single diff
