@@ -1,4 +1,4 @@
-from typing import List, Optional, Dict, Iterator, Callable
+from typing import List, Optional, Dict, Iterator, Callable, Set
 from timeit import default_timer as timer
 import tempfile
 import logging
@@ -13,11 +13,13 @@ from bugzoo.core.coverage import TestSuiteCoverage
 from bugzoo.core.spectra import Spectra
 from bugzoo.localization import SuspiciousnessMetric, Localization
 from bugzoo.testing import TestCase
+from bugzoo.util import indent
 
 import darjeeling.filters as filters
 from .snippet import SnippetDatabase, Snippet
-from .source import SourceFile
-from .util import get_lines
+from .source import SourceFile, SourceFileCollection
+from .util import get_file_contents
+from .exceptions import NoFailingTests, NoImplicatedLines
 
 
 class Problem(object):
@@ -51,6 +53,11 @@ class Problem(object):
                 name appears in the given list. If no list is provided, no
                 filtering of transformations based on the function to which
                 they belong will occur.
+
+        Raises:
+            NoFailingTests: if the program under repair has no failing tests.
+            NoImplicatedLines: if no lines are implicated by the coverage
+                information and the provided suspiciousness metric.
         """
         assert len(in_files) > 0
         self.__bug = bug
@@ -80,22 +87,24 @@ class Problem(object):
             self.__logger.debug("computing coverage information")
             try:
                 container = bz.containers.provision(bug)
-                self.__coverage = bz.coverage.coverage(container, bug.tests)
+                self.__coverage = bz.containers.coverage(container)
             finally:
                 del bz.containers[container.uid]
             self.__logger.debug("computed coverage information")
+        self._dump_coverage()
 
         # restrict coverage information to specified files
         self.__logger.debug("restricting coverage information to files:\n* %s",
                             '\n* '.join(in_files))
         self.__coverage = self.__coverage.restricted_to_files(in_files)
         self.__logger.debug("restricted coverage information.")
+        self._dump_coverage()
+
 
         # determine the passing and failing tests by using coverage information
         self.__logger.debug("using test execution used to generate coverage to determine passing and failing tests")
-
-        self.__tests_failing = set()
-        self.__tests_passing = set()
+        self.__tests_failing = set() # type: Set[TestCase]
+        self.__tests_passing = set() # type: Set[TestCase]
         for test_name in self.__coverage:
             test = bug.harness[test_name]
             test_coverage = self.__coverage[test_name]
@@ -104,10 +113,11 @@ class Problem(object):
             else:
                 self.__tests_failing.add(test)
 
-        # TODO throw an error if there are no failing tests
         self.__logger.info("determined passing and failing tests")
         self.__logger.info("* passing tests: %s", ', '.join([t.name for t in self.__tests_passing]))
         self.__logger.info("* failing tests: %s", ', '.join([t.name for t in self.__tests_failing]))
+        if not self.__tests_failing:
+            raise NoFailingTests
 
         # determine the implicated lines
         # 0. we already restricted to lines that occur in specified files
@@ -120,44 +130,47 @@ class Problem(object):
         if restrict_to_lines is not None:
             self.__lines = self.__lines.intersection(restrict_to_lines)
 
-        # cache contents of implicated files
+        # cache contents of the implicated files
         t_start = timer()
         self.__logger.debug("storing contents of source code files")
-        self.__sources = {} # type: Dict[str, SourceFile]
-        _, fn_host_temp = tempfile.mkstemp(prefix='.darjeeling')
-        ctr_source_files = bz.containers.provision(bug)
-        try:
-            for fn in self.__lines.files:
-                fn_ctr = os.path.join(bug.source_dir, fn)
-                bz.containers.copy_from(ctr_source_files, fn_ctr, fn_host_temp)
-                self.__sources[fn] = SourceFile(fn, get_lines(fn_host_temp))
-            duration = timer() - t_start
-        finally:
-            os.remove(fn_host_temp)
-            del bz.containers[ctr_source_files.uid]
+        self.__sources = SourceFileCollection.from_bug(bz, bug, self.__lines.files)
+        duration = timer() - t_start
         self.__logger.debug("stored contents of source code files (took %.1f seconds)",
                             duration)
 
         # restrict attention to statements
         # FIXME for now, we approximate this -- going forward, we can use
         #   Rooibos to determine transformation targets
+        num_lines_before_filtering = len(self.__lines)
+        self.__logger.debug("filtering lines according to content filters")
         line_content_filters = [
             filters.ends_with_semi_colon,
             filters.has_balanced_delimiters
         ]
         if line_coverage_filters:
-            line_content_filters += line_coverage_filters
-        for fltr_content in line_content_filters:
+            line_content_filters += line_coverage_filters # type: ignore
+        for f in line_content_filters:
             fltr_line = \
-                lambda fl: fltr_content(self.__sources[fl.filename][fl.num])
+                lambda fl: f(self.__sources.line(fl.filename, fl.num))
             self.__lines = self.__lines.filter(fltr_line)
+        num_lines_after_filtering = len(self.__lines)
+        num_lines_removed_by_filtering = \
+            num_lines_after_filtering - num_lines_before_filtering
+        self.__logger.debug("filtered lines according to content files: removed %d lines",  # noqa: pycodestyle
+                            num_lines_removed_by_filtering)
 
         # compute fault localization
+        self.__logger.info("computing fault localization")
         self.__coverage = \
             self.__coverage.restricted_to_files(self.__lines.files)
+        self.__logger.debug("restricted coverage to implicated files")
+        self._dump_coverage()
         self.__spectra = Spectra.from_coverage(self.__coverage)
+        self.__logger.debug("generated coverage spectra: %s", self.__spectra)
         self.__localization = \
             Localization.from_spectra(self.__spectra, suspiciousness_metric)
+        self.__logger.debug("transformed spectra to fault localization: %s",
+                            self.__localization)
         self.__localization = \
             self.__localization.restricted_to_lines(self.__lines)
         self.__logger.info("removing non-suspicious lines from consideration")
@@ -169,14 +182,14 @@ class Problem(object):
         self.__logger.info("removed %d non-suspicious lines from consideration",
                            num_lines_removed)
 
-        # TODO raise an exception if there are no implicated lines
-
         # report implicated lines and files
         self.__logger.info("implicated lines [%d]:\n%s",
                            len(self.__lines), self.__lines)
         self.__logger.info("implicated files [%d]:\n* %s",
                            len(self.__lines.files),
                            '\n* '.join(self.__lines.files))
+        if len(self.__lines) == 0:
+            raise NoImplicatedLines
 
         # construct the snippet database from the parts of the program that
         # were executed by the test suite (both passing and failing tests)
@@ -191,7 +204,7 @@ class Problem(object):
         ]
 
         for line in self.__coverage.lines:
-            content = self.__sources[line.filename][line.num].strip()
+            content = self.__sources.line(line.filename, line.num).strip()
             if all(fltr(content) for fltr in snippet_filters):
                 # self.__logger.debug("* found snippet at %s: %s", line, content)
                 snippet = Snippet(content)
@@ -207,11 +220,15 @@ class Problem(object):
         self.__coverage.restricted_to_files(self.__lines.files)
         self.__spectra.restricted_to_files(self.__lines.files)
         extraneous_source_fns = \
-            set(self.__sources.keys()) - set(self.__lines.files)
+            set(self.__sources.files) - set(self.__lines.files)
         for fn in extraneous_source_fns:
-            del self.__sources[fn]
+            self.__sources = self.__sources.without_file(fn)
         self.__logger.debug("finished reducing memory footprint")
 
+    def _dump_coverage(self) -> None:
+        self.__logger.debug("[COVERAGE]\n%s\n[/COVERAGE]",
+                            indent(str(self.__coverage), 2))
+
     @property
     def snippets(self) -> SnippetDatabase:
         """
@@ -289,11 +306,12 @@ class Problem(object):
         """
         return self.__lines.__iter__()
 
-    def source(self, fn: str) -> SourceFile:
+    @property
+    def sources(self) -> SourceFileCollection:
         """
-        Returns the contents of a given source code file for this problem.
+        The source code files for the program under repair.
         """
-        return self.__sources[fn]
+        return self.__sources
 
     def check_sanity(self,
                      expected_to_pass: List[TestCase],
