@@ -16,15 +16,17 @@ from .generator import DeletionGenerator, \
                        AllTransformationsAtLine, \
                        SampleByLocalization
 
+logger = logging.getLogger(__name__)
 
-__ALL__ = ['RepairReport', 'repair']
+
+__all__ = ['RepairReport', 'repair']
 
 
 class RepairReport(object):
     def __init__(self,
                  num_candidate_evals: int,
                  num_test_evals: int,
-                 duration: timedelta):
+                 duration: timedelta) -> None:
         self.__num_candidate_evals = num_candidate_evals
         self.__num_test_evals = num_test_evals
         self.__duration = duration
@@ -54,7 +56,7 @@ class RepairReport(object):
 def repair(bugzoo: bugzoo.BugZoo,
            problem: Problem,
            seed: Optional[int] = None,
-           threads: Optional[int] = 1,
+           threads: int = 1,
            terminate_early: Optional[bool] = True,
            time_limit: Optional[timedelta] = None,
            logger: Optional[logging.Logger] = None
@@ -71,7 +73,8 @@ def repair(bugzoo: bugzoo.BugZoo,
     # transformations = \
     #     AllTransformationsAtLine(line, problem.snippets)
     transformations = \
-        SampleByLocalization(problem.localization,
+        SampleByLocalization(problem,
+                             problem.localization,
                              problem.snippets,
                              randomize=False)
     candidates = SingleEditPatches(transformations)
@@ -80,8 +83,7 @@ def repair(bugzoo: bugzoo.BugZoo,
                         problem,
                         candidates,
                         threads=threads,
-                        time_limit=time_limit,
-                        logger=logger)
+                        time_limit=time_limit)
 
     if terminate_early:
         try:
@@ -92,6 +94,17 @@ def repair(bugzoo: bugzoo.BugZoo,
     else:
         repairs = list(searcher)
 
+    #for candidate in searcher.outcomes:
+    #    outcome = searcher.outcomes[candidate]
+    #    if not outcome.build.successful:
+    #        print("FAILED TO COMPILE: {}".format(candidate))
+    #    else:
+    #        try:
+    #            failed = next(t for t in outcome.tests if not outcome.tests[t].successful)
+    #            print("FAILED TEST ({}): {}".format(failed, candidate))
+    #        except StopIteration:
+    #            print("SUCCESS: {}".format(candidate))
+
     report = RepairReport(searcher.num_candidate_evals,
                           searcher.num_test_evals,
                           searcher.time_running)
